function [dataset] = prepare_dataset(dataset_name, labels_provided, clusterer_name, delete_files)

% Input:
% dataset_name: is the path along with the name of the data set
% labels_provided: does the last column represent class labels?
% clusterer_name: clustering method, either Xmeans or EM
% delete_files: delete temporary files?
%
% Output: 
% dataset is simply the dataset stored in dataset_name file

%% Extract 1D centroids from shell script output file

pattern1 = '[-+]?\d+\.?\d*([eE][-+]?[0-9]+)?';

if(strcmp(dataset_name(end-2:end), 'csv'))
	dataset = csvread(dataset_name);
elseif(strcmp(dataset_name(end-2:end), 'dlm'))
	dataset = dlmread(dataset_name, ' ');
end

[n_inst, n_attr] = size(dataset);
if(labels_provided)
	n_attr = n_attr - 1;
end

file_1 = [dataset_name(1:end-4), '-resultsby-', clusterer_name];
file_2 = [dataset_name(1:end-4), '-AttributeWiseClusterCentroids_temp'];

fid = fopen(file_1);
fid2 = fopen(file_2, 'w');

if(strcmp(clusterer_name,'Xmeans'))
	
	for(j = 1:n_attr)
		
		tline = fgetl(fid);
		while(size(tline,2)<40 || ~strcmp(tline(1:15),'Cluster centers'))
			tline = fgetl(fid);
		end
		
		[data] = regexp(tline, pattern1, 'match');
		% count of centroids in current attribute
		cent_count = str2num(data{1});
		fprintf(fid2, '%u', cent_count);
		% blank line
		tline = fgetl(fid);
		
		for(i = 1:cent_count)
			fprintf(fid2, ' ');
			tline = fgetl(fid);
			tline = fgetl(fid);
			[data] = regexp(tline, pattern1, 'match');
			fprintf(fid2, data{1});
		end
		
		fprintf(fid2, '\n');
		% centroids are all written to file fid2
		
	end % while or for
	
elseif(strcmp(clusterer_name,'EM'))
	
	for(j = 1:n_attr)
		
		tline = fgetl(fid);
		while(size(tline,2)<14 || ~strcmp(tline(3:6),'mean'))
			tline = fgetl(fid);
		end
		
		[data] = regexp(tline, pattern1, 'match');
		% count of centroids in current attribute
		cent_count = length(data);
		fprintf(fid2, '%u', cent_count);
		
		for(i = 1:cent_count)
			a = str2num(data{i});
			fprintf(fid2, '% .4f', a);
		end
		
		fprintf(fid2, '\n');
		% centroids are all written to file fid2
		
	end % while or for
	
end % (clusterer_name == 'Xmeans')

fclose(fid);
fclose(fid2);

%% Calculate counts, std devs, noneligible attributes, and centroids

fn_p1 = [dataset_name(1:end-4), '-attr'];
fn_p2 = ['-clusteredby-', clusterer_name];

cents = dlmread(file_2);
[counts, stddevs] = std_devs(dataset, labels_provided, cents, fn_p1, fn_p2);
[non_attr, non_cent, non_cent2] = noneligible_attributes_centroids(dataset, labels_provided, cents, stddevs, counts);

if(labels_provided)
	labels = dataset(:,end);
	dlmwrite([dataset_name(1:end-4), '-Labels'], labels, 'delimiter', ' ');
end

dlmwrite([dataset_name(1:end-4), '-NoneligibleAttributes'], non_attr, 'delimiter', ' ');
dlmwrite([dataset_name(1:end-4), '-Noneligible1DClusterCentroids_temp'], non_cent, 'delimiter', ' ');

%% Sort cluster centroids, cluster labels, and noneligible cluster centroids

format long

cents2 = -1*ones(size(cents));
non_cent2 = zeros(size(non_cent));

fn_p1 = [dataset_name(1:end-4), '-attr'];
fn_p2 = ['-clusteredby-', clusterer_name];

% === added ===
sorted_labels = -1*ones(n_inst, n_attr);
file_3 = [dataset_name(1:end-4), '-AttributeWiseClusterCentroids'];
fid3 = fopen(file_3, 'w');
% === added ===

for(i = 1:size(cents,1))
	if(mod(i,100)==0)
		i
	end
	cents2(i,1) = cents(i,1);
	fprintf(fid3, '%u', cents2(i,1));
	[cents2(i,2:cents(i,1)+1), idx] = sort(cents(i,2:cents(i,1)+1));

	for(j= 2:cents2(i,1)+1)
		fprintf(fid3, '% .4f', cents2(i,j));
	end
	fprintf(fid3, '\n');
	filename = [fn_p1, num2str(i-1), fn_p2];
	l = dlmread(filename);
	l = l(:,2);
	l2 = -1*ones(size(l));
	for(j=1:cents(i,1))
		l2(l==idx(j)-1) = j-1;
		non_cent2(i,j) = non_cent(i,idx(j));
	end
% 	l3 = [0:length(l2)-1]';
% 	l2 = [l3 l2];

% === added ===
	sorted_labels(:,i) = l2;
% === added ===

% 	outFile = [fn_p1, num2str(i-1), fn_p2, '_sorted'];
% 	dlmwrite(outFile, l2, 'delimiter', ' ');
end

% === added ===
dlmwrite([dataset_name(1:end-4), '-AttributeWiseClusterLabels'], sorted_labels, 'delimiter', ' ');
% === added ===

% dlmwrite([dataset_name(1:end-4), '-AttributeWiseClusterCentroids_sorted'], cents2, 'delimiter', ' ', 'precision', '%.4f');
dlmwrite([dataset_name(1:end-4), '-Noneligible1DClusterCentroids'], non_cent2, 'delimiter', ' ');

format short

%% Merge all cluster labels in one file

% merged = -1*ones(n_inst, n_attr);
% 
% fn_p1 = [dataset_name(1:end-4), '-attr'];
% fn_p2 = ['-clusteredby-', clusterer_name];
% 
% for(i = 1:n_attr)
% 	filename = [fn_p1, num2str(i-1), fn_p2];
% 	l = dlmread(filename);
% 	l = l(:,2);
% 	merged(:,i) = l;
% end
% 
% dlmwrite([dataset_name(1:end-4), '-AttributeWiseClusterLabels_sorted'], merged, 'delimiter', ' ');

%% Deleting all unnecessary 1D lables files

if(delete_files)
	% delete atribute-wise files
	delete([dataset_name(1:end-4), '-attr*']);
	% delete 2 temporary files 
	delete(['*', '_temp']);
	% delete the output file of Xmeans or EM 'dataset-resultsby-clusterer'
	delete([dataset_name(1:end-4), '-resultsby-', clusterer_name])
end

end % function
 